#ifdef SESOMA_MYSQL

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql.h>

static MYSQL *myh = NULL;

static char error[512];

static char *ihost, *idb, *iuser, *ipass, *isock;
static int iport;
static int mysql_connected = 0;

char *sesoma_mysql_strerror(void)
{
	return error;
}

/* returns 0 if it's ok */
int sesoma_mysql_connect(char *host, char *port, char *user, char *pass, char *db, char *sock)
{
	MYSQL *p;

	ihost = host;
	idb = db;
	iuser = user;
	ipass = pass;
	isock = sock;
	if (port)
		iport = strtol(port, NULL, 10);
	else
		iport = 3306;

	myh = mysql_init(NULL);
	if (!myh) {
		snprintf(error, sizeof(error),
			"Error calling mysql_init!");
		return 1;
	}

	p = mysql_real_connect(myh, ihost, iuser, ipass, idb, iport, isock, 0);
	if (!p) {
		snprintf(error, sizeof(error),
			"Cannot connect to db server!");
		return 1;
	}

	mysql_connected = 1;

	return 0;
}

/* reconnect to server */
static int sesoma_mysql_reconnect(void)
{
	MYSQL *p;

	if (mysql_connected == 1)
		return 0;

	p = mysql_real_connect(myh, ihost, iuser, ipass, idb, iport, isock, 0);
	if (!p) {
		snprintf(error, sizeof(error),
			"Cannot reconnect to db server!");
		return 1;
	}

	mysql_connected = 1;

	return 0;
}

/* ret is 0 => found */
/* ret is 1 => not found */
/* ret is 2 => temporary error */
/* ret is 3 => unexpected error */
char *sesoma_mysql_query(char *query, int *ret)
{
	char *s = NULL;
	int rows, re;
	MYSQL_RES *res;
	MYSQL_ROW row;

	strcpy(error, "OK");

	re = sesoma_mysql_reconnect();
	if (re != 0) {
		snprintf(error, sizeof(error),
			"Connection lost with db server. I will reconnect!");
		*ret = 2;
		goto out2;
	}

	re = mysql_real_query(myh, query, strlen(query));
	if (re != 0) {
		snprintf(error, sizeof(error),
			"Cannot do the query.");
		mysql_connected = 0;
		*ret = 2;
		goto out2;
	}

	res = mysql_store_result(myh);
	if (!res) {
		snprintf(error, sizeof(error),
			"Cannot store result!");
		*ret = 2;
		goto out;
	}

	rows = mysql_num_rows(res);
	if (rows == 0) {
		*ret = 1;
		goto out;
	}

	row = mysql_fetch_row(res);
	if (!row) {
		snprintf(error, sizeof(error),
			"Cannot fetch row!");
		*ret = 2;
		goto out;
	}

	*ret = 0;
	s = row[0];

	out:
	mysql_free_result(res);

	out2:
	return s;
}

void sesoma_mysql_escape(char *to, char *from, unsigned int from_len)
{
	mysql_escape_string(to, from, from_len);
}

#else
	/* mysql not selected */
	short sesoma_mysql = 0;
#endif
