#ifdef SESOMA_PGSQL

#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <libpq-fe.h>

static PGconn *conn = NULL;

static char error[512];

static char connstr[1024];
static int pgsql_connected = 0;

char *sesoma_pgsql_strerror(void)
{
	return error;
}

/* returns 0 if it's ok */
int sesoma_pgsql_connect(char *host, char *port, char *user, char *pass, char *db, char *junk)
{
	char *add = "";
	char tmp[128];
	int len = 0, len2, total;

	total = sizeof(connstr);
	strcpy(connstr, "");

	if (host != NULL) {
		len2 = snprintf(tmp, sizeof(tmp), "%shost=%s", add, host);
		if (len + len2 < total)
			strcat(connstr, tmp);
		len += len2;
		add = " ";
	}

	if (port != NULL) {
		len2 = snprintf(tmp, sizeof(tmp), "%sport=%s", add, port);
		if (len + len2 < total)
			strcat(connstr, tmp);
		len += len2;
		add = " ";
	}

	if (db != NULL) {
		len2 = snprintf(tmp, sizeof(tmp), "%sdbname=%s", add, db);
		if (len + len2 < total)
			strcat(connstr, tmp);
		len += len2;
		add = " ";
	}

	if (user != NULL) {
		len2 = snprintf(tmp, sizeof(tmp), "%suser=%s", add, user);
		if (len + len2 < total)
			strcat(connstr, tmp);
		len += len2;
		add = " ";
	}

	if (pass != NULL) {
		len2 = snprintf(tmp, sizeof(tmp), "%spassword=%s", add, pass);
		if (len + len2 < total)
			strcat(connstr, tmp);
		len += len2;
		add = " ";
	}

	conn = PQconnectdb(connstr);
	if (PQstatus(conn) != CONNECTION_OK) {
		snprintf(error, sizeof(error), "Cannot connect to postgresql server connstr=(%s)",
			connstr);
		return 1;
	}

	pgsql_connected = 1;

	return 0;
}

/* reconnect to server */
static int sesoma_pgsql_reconnect(void)
{
	if (pgsql_connected == 1)
		return 0;

	PQfinish(conn);

	conn = PQconnectdb(connstr);
	if (PQstatus(conn) != CONNECTION_OK) {
		snprintf(error, sizeof(error), "Cannot (re)connect to db server connstr=(%s)",
			connstr);
		return 1;
	}

	pgsql_connected = 1;

	return 0;
}

/* ret is 0 => found */
/* ret is 1 => not found */
/* ret is 2 => temporary error */
/* ret is 3 => unexpected error */
char *sesoma_pgsql_query(char *query, int *ret)
{
	PGresult *res;
	ExecStatusType stat;
	char *s = NULL;
	int rows, re;

	strcpy(error, "OK");

	re = sesoma_pgsql_reconnect();
	if (re != 0) {
		snprintf(error, sizeof(error),
			"Connection lost with db server. I will reconnect!");
		*ret = 2;
		goto out2;
	}

	res = PQexec(conn, query);
	if (!res) {
		snprintf(error, sizeof(error),
			"Cannot do the query. Probably out of memory!");
		pgsql_connected = 0;
		*ret = 2;
		goto out2;
	}

	stat = PQresultStatus(res);

	if (stat == CONNECTION_BAD) {
		snprintf(error, sizeof(error),
			"Connection lost with db server. I will reconnect!");
		pgsql_connected = 0;
		*ret = 2;
		goto out;
	}

	if ((stat != PGRES_COMMAND_OK) && (stat != PGRES_TUPLES_OK)) {
		snprintf(error, sizeof(error),
			"%s", PQresStatus(stat));
		*ret = 2;
		goto out;
	}

	rows = PQntuples(res);
	if (rows == 0) {
		*ret = 1;
		goto out;
	}

	if (stat == PGRES_TUPLES_OK) {
		*ret = 0;
		s = PQgetvalue(res, 0, 0);
		goto out;
	}

	*ret = 3;

	out:
	PQclear(res);

	out2:
	return s;
}

void sesoma_pgsql_escape(char *to, char *from, unsigned int from_len)
{
	PQescapeString(to, from, from_len);
}

#else
	/* pgsql not selected */
	short sesoma_pgsql = 0;
#endif
